@@ -883,11 +883,8 @@ namespace Concurrency { namespace streams
                     if (ch == concurrency::streams::char_traits<CharType>::eof()) return pplx::task_from_result(false);
                     if (ch == '\n')
                     {
-						return buffer.bumpc().then([](
-#ifndef _WIN32 // Required by GCC
-							typename
-#endif
-							concurrency::streams::char_traits<CharType>::int_type) { return false; });
+                        return buffer.bumpc().then([](
+                            typename concurrency::streams::char_traits<CharType>::int_type) { return false; });
                     }
                     return pplx::task_from_result(false);
                 };
@@ -896,10 +893,7 @@ namespace Concurrency { namespace streams
                 {
                     while ( buffer.in_avail() > 0 )
                     {
-#ifndef _WIN32 // Required by GCC, because concurrency::streams::char_traits<CharType> is a dependent scope
-                        typename
-#endif
-                        concurrency::streams::char_traits<CharType>::int_type ch;
+                        typename concurrency::streams::char_traits<CharType>::int_type ch;
 
                         if (_locals->saw_CR)
                         {
@@ -1717,55 +1711,11 @@ private:
 };
 
 #ifdef _WIN32
-template<>
-class type_parser<char,std::basic_string<wchar_t>> : public _type_parser_base<char>
+template<class CharType>
+class type_parser<CharType, std::enable_if_t<sizeof(CharType) == 1, std::basic_string<wchar_t>>> : public _type_parser_base<CharType>
 {
 public:
-    static pplx::task<std::wstring> parse(streams::streambuf<char> buffer)
-    {
-        return _parse_input<std::basic_string<char>,std::basic_string<wchar_t>>(buffer, _accept_char, _extract_result);
-    }
-
-private:
-    static bool _accept_char(const std::shared_ptr<std::basic_string<char>> &state, int_type ch)
-    {
-        if ( ch == concurrency::streams::char_traits<char>::eof() || isspace(ch)) return false;
-        state->push_back(char(ch));
-        return true;
-    }
-    static pplx::task<std::basic_string<wchar_t>> _extract_result(std::shared_ptr<std::basic_string<char>> state)
-    {
-        return pplx::task_from_result(utility::conversions::utf8_to_utf16(*state));
-    }
-};
-
-template<>
-class type_parser<signed char,std::basic_string<wchar_t>> : public _type_parser_base<signed char>
-{
-public:
-    static pplx::task<std::wstring> parse(streams::streambuf<signed char> buffer)
-    {
-        return _parse_input<std::basic_string<char>,std::basic_string<wchar_t>>(buffer, _accept_char, _extract_result);
-    }
-
-private:
-    static bool _accept_char(const std::shared_ptr<std::basic_string<char>> &state, int_type ch)
-    {
-        if ( ch == concurrency::streams::char_traits<char>::eof() || isspace(ch)) return false;
-        state->push_back(char(ch));
-        return true;
-    }
-    static pplx::task<std::basic_string<wchar_t>> _extract_result(std::shared_ptr<std::basic_string<char>> state)
-    {
-        return pplx::task_from_result(utility::conversions::utf8_to_utf16(*state));
-    }
-};
-
-template<>
-class type_parser<unsigned char,std::basic_string<wchar_t>> : public _type_parser_base<unsigned char>
-{
-public:
-    static pplx::task<std::wstring> parse(streams::streambuf<unsigned char> buffer)
+    static pplx::task<std::wstring> parse(streams::streambuf<CharType> buffer)
     {
         return _parse_input<std::basic_string<char>,std::basic_string<wchar_t>>(buffer, _accept_char, _extract_result);
     }
@@ -1787,3 +1737,4 @@ private:
 }}
 
 #endif
+
