@@ -15,6 +15,7 @@
 #include "stdafx.h"
 
 #include "http_client_impl.h"
+#include "../common/internal_http_helpers.h"
 
 #include <Strsafe.h>
 // Important for WP8
@@ -82,25 +83,33 @@ public:
 
         utf16char *hdrStr = nullptr;
         HRESULT hr = xmlReq->GetAllResponseHeaders(&hdrStr);
-        if(hr != S_OK)
+        if (SUCCEEDED(hr))
         {
-            return hr;
-        }
+            try
+            {
+                auto progress = m_request->m_request._get_impl()->_progress_handler();
+                if (progress && m_request->m_uploaded == 0)
+                {
+                    (*progress)(message_direction::upload, 0);
+                }
 
-        auto progress = m_request->m_request._get_impl()->_progress_handler();
-        if (progress && m_request->m_uploaded == 0)
-        {
-            try { (*progress)(message_direction::upload, 0); } catch(...)
+                web::http::details::parse_headers_string(hdrStr, response.headers());
+                m_request->complete_headers();
+            }
+            catch (...)
             {
                 m_request->m_exceptionPtr = std::current_exception();
-                return ERROR_UNHANDLED_EXCEPTION;
+                hr = ERROR_UNHANDLED_EXCEPTION;
             }
         }
 
-        web::http::details::parse_headers_string(hdrStr, response.headers());
-        m_request->complete_headers();
+        if (hdrStr != nullptr)
+        {
+            ::CoTaskMemFree(hdrStr);
+            hdrStr = nullptr;
+        }
 
-        return S_OK;
+        return hr;
     }
 
     // Called when a portion of the entity body has been received.
@@ -434,13 +443,13 @@ protected:
             if (client_cred.is_set())
             {
                 username = client_cred.username();
-                password_plaintext = client_cred.decrypt();
+                password_plaintext = client_cred._internal_decrypt();
                 password = password_plaintext->c_str();
             }
             if (proxy_cred.is_set())
             {
                 proxy_username = proxy_cred.username();
-                proxy_password_plaintext = proxy_cred.decrypt();
+                proxy_password_plaintext = proxy_cred._internal_decrypt();
                 proxy_password = proxy_password_plaintext->c_str();
             }
 
