@@ -22,6 +22,9 @@
 #include "http_server_httpsys.h"
 #include "http_server_impl.h"
 
+#undef min
+#undef max
+
 using namespace web;
 using namespace utility;
 using namespace concurrency;
@@ -1054,4 +1057,4 @@ std::unique_ptr<http_server> make_http_httpsys_server()
 
 }}}}
 
-#endif
\ No newline at end of file
+#endif
